@@ -9,7 +9,6 @@ from queue import Queue
 import socket
 import threading
 from threading import Thread, Event
-import _thread
 from datetime import datetime
 import time
 import traceback
@@ -177,7 +176,8 @@ class TransmissionListener:
     def __init__(self, listener_name, eventhandler):
         self.listener_name = listener_name
         self.eventhandler = eventhandler
-        self.listener = Thread(target=self.Listen, args=())
+        self.listener = Thread(target=self.Listen, args=(),
+                               name=f"DataTransmissionListener-{listener_name}")
         self.listener.start()
 
     def ReceiveTransmissionRequests(self, data):
@@ -212,7 +212,7 @@ class TransmissionListener:
             sock.listen()
 
             listener = Thread(target=self.ReceiveTransmission, args=(
-                peerID, sock, our_port, self.eventhandler))
+                peerID, sock, our_port, self.eventhandler), name=f"DataTransmissionReceiver-{our_port}")
             listener.start()
             return our_port
 
@@ -243,7 +243,8 @@ class TransmissionListener:
         data = recv_timeout(conn)
         conn.send("Finished!".encode())
         # conn.close()
-        _thread.start_new_thread(eventhandler, (data, peerID))
+        Thread(target=eventhandler, args=(data, peerID),
+               name="TransmissionListener.ReceivedTransmission").start()
         CloseListeningConnection(str(our_port), our_port)
         sock.close()
 
@@ -411,6 +412,7 @@ class Conversation:
     listener = None
     __encryption_callback = None
     __decryption_callback = None
+    _terminate = False
 
     def __init__(self):
         self.started = Event()
@@ -462,7 +464,7 @@ class Conversation:
             bool success: whether or not the conversation with the peer was successfully started
         """
         if peerID == IPFS_API.MyID():
-            return
+            return False
         if(print_log_conversations):
             print(conversation_name + ": Starting conversation")
         self.conversation_name = conversation_name
@@ -480,7 +482,7 @@ class Conversation:
         self.listener = ListenForTransmissions(conversation_name,
                                                self.Hear
                                                )
-        self.file_receiver = ListenForFileTransmissions(
+        self.file_listener = ListenForFileTransmissions(
             f"{conversation_name}:files",
             self.FileReceived,
             progress_handler=self.file_progress_callback,
@@ -546,7 +548,7 @@ class Conversation:
         self.listener = ListenForTransmissions(conversation_name,
                                                self.Hear,
                                                )
-        self.file_receiver = ListenForFileTransmissions(
+        self.file_listener = ListenForFileTransmissions(
             f"{conversation_name}:files", self.FileReceived,
             progress_handler=self.file_progress_callback,
             dir=dir,
@@ -575,6 +577,8 @@ class Conversation:
         Forwards it to the user's data_received_eventhandler if the conversation has already started,
         otherwise processes the converation initiation codes.
         """
+        if self._terminate:
+            return
         # print("HEAR", data)
         if not data:
             print("CONV.HEAR: RECEIVED NONE")
@@ -605,9 +609,11 @@ class Conversation:
             if self.data_received_eventhandler:
                 # if the data_received_eventhandler has 2 parameters
                 if len(signature(self.data_received_eventhandler).parameters) == 2:
-                    Thread(target=self.data_received_eventhandler, args=(self, data)).start()
+                    Thread(target=self.data_received_eventhandler,
+                           args=(self, data), name="Converstion.data_received_eventhandler").start()
                 else:
-                    Thread(target=self.data_received_eventhandler, args=(self, data, arg3)).start()
+                    Thread(target=self.data_received_eventhandler, args=(
+                        self, data, arg3), name="Converstion.data_received_eventhandler").start()
 
     def Listen(self, timeout=None):
         """Waits until the conversation peer sends a message,
@@ -615,6 +621,8 @@ class Conversation:
         Can be used as an alternative to specifying an data_received_eventhandler
             to process received messages, or in parallel.
         """
+        if self._terminate:
+            return
         if not timeout:
             data = self.message_queue.get()
         else:
@@ -635,7 +643,8 @@ class Conversation:
             print(f"{self.conversation_name}: Received file: ", filepath)
         self.file_queue.put({'filepath': filepath, 'metadata': metadata})
         if self.file_eventhandler:
-            _thread.start_new_thread(self.file_eventhandler, (self, filepath, metadata))
+            Thread(target=self.file_eventhandler, args=(self, filepath, metadata),
+                   name='Conversation.file_eventhandler').start()
 
     def ListenForFile(self, timeout=None):
         if not timeout:
@@ -712,9 +721,15 @@ class Conversation:
             transmission_send_timeout_sec=transmission_send_timeout_sec,
             transmission_request_max_retries=transmission_request_max_retries)
 
-    def Close(self):
+    def Terminate(self):
+        self._terminate = True
         if self.listener:
             self.listener.Terminate()
+        if self.file_listener:
+            self.file_listener.Terminate()
+
+    def Close(self):
+        self.Terminate()
 
     def __del__(self):
         self.Close()
@@ -980,12 +995,14 @@ class FileTransmitter:
     def CallProgressCallBack(self, progress):
         if self.progress_handler:
             if len(signature(self.progress_handler).parameters) == 1:
-                _thread.start_new_thread(self.progress_handler, (progress,))
+                Thread(target=self.progress_handler, args=(progress,),
+                       name='Conversation.progress_handler').start()
             elif len(signature(self.progress_handler).parameters) == 2:
-                _thread.start_new_thread(self.progress_handler, (self.filename, progress))
+                Thread(target=self.progress_handler, args=(self.filename, progress),
+                       name='Conversation.progress_handler').start()
             elif len(signature(self.progress_handler).parameters) == 3:
-                _thread.start_new_thread(self.progress_handler,
-                                         (self.filename, self.filesize, progress))
+                Thread(target=self.progress_handler,
+                       args=(self.filename, self.filesize, progress), name='Conversation.progress_handler').start()
 
     def Hear(self, conv, data):
         if print_log_files:
@@ -1043,8 +1060,8 @@ class FileTransmissionReceiver:
                     print("FileReception: " + self.filename
                           + ": ready to receive file")
                 if self.progress_handler:
-                    _thread.start_new_thread(self.progress_handler,
-                                             (self.conv.peerID, self.filename, self.filesize, 0))
+                    Thread(target=self.progress_handler, args=(self.conv.peerID, self.filename,
+                           self.filesize, 0), name='FileTransmissionReceiver.progress_handler').start()
 
                 if(self.filesize == 0):
                     self.Finish()
@@ -1058,8 +1075,8 @@ class FileTransmissionReceiver:
             self.writtenbytes += len(data)
 
             if self.progress_handler:
-                _thread.start_new_thread(self.progress_handler,
-                                         (self.conv.peerID, self.filename, self.filesize, self.writtenbytes/self.filesize))
+                Thread(target=self.progress_handler,
+                       args=(self.conv.peerID, self.filename, self.filesize, self.writtenbytes/self.filesize), name='FileTransmissionReceiver.progress').start()
 
             if print_log_files:
                 print("FileTransmission: " + self.filename
@@ -1089,6 +1106,9 @@ class FileTransmissionReceiver:
             else:
                 self.eventhandler(self.conv.peerID, filepath)
 
+    def Terminate(self):
+        self.conv.Terminate()
+
 
 # ----------IPFS Technicalitites-------------------------------------------
 connections_send = list()
@@ -1243,6 +1263,7 @@ class ListenerTCP(threading.Thread):
     def __init__(self, eventhandler, port=0, buffer_size=def_buffer_size, monitoring_interval=2, status_eventhandler=None, eventhandlers_on_new_threads=True):
         threading.Thread.__init__(self)
         self.port = port
+        self.name = f"TCPListener-{port}"
         self.eventhandler = eventhandler
         self.buffer_size = buffer_size
         self.eventhandlers_on_new_threads = eventhandlers_on_new_threads
@@ -1257,8 +1278,8 @@ class ListenerTCP(threading.Thread):
             self.status_eventhandler = status_eventhandler
             self.monitoring_interval = monitoring_interval
             self.last_time_recv = datetime.utcnow()
-            self.status_monitor_thread = _thread.start_new_thread(
-                self.StatusMonitor, ())
+            self.status_monitor_thread = Thread(
+                target=self.StatusMonitor, args=(), name='ListenerTP.status_monitor').start()
 
         self.start()
 
@@ -1282,7 +1303,8 @@ class ListenerTCP(threading.Thread):
                 # break
             if len(data) > 0:
                 if self.eventhandlers_on_new_threads:
-                    ev = Thread(target=self.eventhandler, args=(data, ))
+                    ev = Thread(target=self.eventhandler, args=(
+                        data, ), name="TCPListener-eventhandler")
                     ev.start()
                 else:
                     self.eventhandler(data)
@@ -1346,6 +1368,7 @@ class Listener2TCP(threading.Thread):
     def __init__(self, eventhandler, port=0, buffer_size=def_buffer_size):
         threading.Thread.__init__(self)
         self.port = port
+        self.name = f"TCPListener-{port}"
         self.eventhandler = eventhandler
         self.buffer_size = buffer_size
 
@@ -1379,7 +1402,8 @@ class Listener2TCP(threading.Thread):
             if print_log_connections:
                 print("Received data")
             if len(data) > 0:
-                ev = Thread(target=self.eventhandler, args=(data, ip_addr))
+                ev = Thread(target=self.eventhandler, args=(
+                    data, ip_addr), name="TCPListener-eventhandler")
                 ev.start()
         conn.close()
 
@@ -1430,6 +1454,7 @@ class Listener(threading.Thread):
     def __init__(self, eventhandler, port=0, buffer_size=def_buffer_size, monitoring_interval=2, status_eventhandler=None):
         threading.Thread.__init__(self)
         self.port = port
+        self.name = f"Listener-{port}"
         self.eventhandler = eventhandler
         self.buffer_size = buffer_size
 
@@ -1444,8 +1469,8 @@ class Listener(threading.Thread):
             self.status_eventhandler = status_eventhandler
             self.monitoring_interval = monitoring_interval
             self.last_time_recv = datetime.utcnow()
-            self.status_monitor_thread = _thread.start_new_thread(
-                self.StatusMonitor, ())
+            self.status_monitor_thread = Thread(
+                target=self.StatusMonitor, args=(), name='Listener.status_monitor').start()
 
         self.start()
 
@@ -1468,7 +1493,8 @@ class Listener(threading.Thread):
             if len(data) > 0:
                 if print_log_connections:
                     print("received buffer")
-                ev = Thread(target=self.eventhandler, args=(data, ip_addr))
+                ev = Thread(target=self.eventhandler, args=(
+                    data, ip_addr), name="Listener-eventhandler")
                 ev.start()
             else:
                 if print_log_connections:
@@ -1534,6 +1560,7 @@ class Listener2(threading.Thread):
     def __init__(self, eventhandler, port=0, buffer_size=def_buffer_size):
         threading.Thread.__init__(self)
         self.port = port
+        self.name = f"Listener2-{port}"
         self.eventhandler = eventhandler
         self.buffer_size = buffer_size
 
@@ -1565,7 +1592,8 @@ class Listener2(threading.Thread):
             if print_log_connections:
                 print("Received data")
             if len(data) > 0:
-                ev = Thread(target=self.eventhandler, args=(data, ip_addr))
+                ev = Thread(target=self.eventhandler, args=(
+                    data, ip_addr), name="Listener2-eventhandler")
                 ev.start()
 
         # _thread.start_new_thread(eventhandler, (data, peerID))
